@@ -53,7 +53,7 @@ def escape_csv_string(string):
 
 
 def validate_headers(headers):
-    return headers != None and all({k.strip() == headers[k].strip(): headers[k] for k in headers})
+    return headers is not None and all({k is not None and headers[k] is not None and k.strip() == headers[k].strip(): headers[k] for k in headers})
 
 
 def check_row(row):
@@ -64,11 +64,13 @@ class Subtask:
     name: str
     max_points: float
     half_points: float
+    is_bonus: bool
 
-    def __init__(self, name: str, max_points: float):
+    def __init__(self, name: str, max_points: float, is_bonus: bool):
         self.name = name
         self.max_points = max_points
         self.half_points = max_points / 2
+        self.is_bonus = is_bonus
 
 
 class Ep2Group:
@@ -290,7 +292,8 @@ class Ep2:
         try:
             git.Git(repo_dir).checkout(tag)  # checkout exercise tag
         except git.exc.GitCommandError as error:
-            return Err('%s failed with status code: %d (%s)' % (error.command, error.status, error.stdout))
+            return Err('%s failed with status code: %d (stdout: %s) (stderr: %s)' % (error.command, error.status,
+                                                                                     error.stdout, error.stderr))
         return Ok()
 
     def ue_repo(self, mat_no):
@@ -417,6 +420,30 @@ class Ep2:
                     click.echo('System error: {}'.format(e))
                     exit(5)
 
+    def local_team_repository(self, team: str):
+        return os.path.join(self.uebung_dir(), team)
+
+    def create_ex_test_symlink(self, project: str, team: str):
+        local_team = self.local_team_repository(team)
+        local_repo = self.local_repo(project)
+
+        if os.path.exists(local_team):
+            try:
+                os.unlink(local_team)
+            except IsADirectoryError:
+                try:
+                    os.rmdir(local_team)
+                except OSError:
+                    shutil.rmtree(local_team)
+
+        try:
+            os.symlink(local_repo, local_team)
+        except OSError as e:
+            click.echo(
+                'User has no rights to create symlinks! See https://stackoverflow.com/a/65504258 to set proper '
+                'privileges under Windows 10.')
+            click.echo('System error: {}'.format(e))
+
     def test_info_csv(self) -> str:
         return os.path.join(self.tutor_repo(), "test_info.csv")
 
@@ -476,7 +503,7 @@ class Ep2:
         csv_file = self.exercise_subtasks_csv(ex)
 
         with open(csv_file, 'r') as infile:
-            reader = csv.DictReader(infile, ['subtask', 'max_points'], KEY_INVALID, strict=True)
+            reader = csv.DictReader(infile, ['subtask', 'max_points', 'bonus'], KEY_INVALID, strict=True)
 
             headers = next(reader, None)
             if not validate_headers(headers):
@@ -485,7 +512,7 @@ class Ep2:
 
             subtasks: [Subtask] = []
             for row in reader:
-                subtasks += [Subtask(row['subtask'], float(row['max_points']))]
+                subtasks += [Subtask(row['subtask'], float(row['max_points']), row['bonus'] == 'true')]
 
             return subtasks
 
