@@ -308,7 +308,9 @@ def submission(ctx, group, ue, late, idea, grade, single):
                         clone_result = ep2.clone_or_update(current_uebung, ep2.ue_repo(repo_owner), TAG_NAME_EX_TEST % ue)
                         if clone_result.is_err():
                             click.secho("Could not checkout submission from {} for team {}, {}"
-                                        .format(student_id, row[KEY_EX_TEST_TEAM], clone_result.value), fg='red')
+                                        .format(repo_owner, team.name, clone_result.value), fg='red')
+                            f_info.print_info()
+                            exit(1)
                         else:
                             if idea:
                                 ep2.idea_checkout(repo_owner)
@@ -318,6 +320,8 @@ def submission(ctx, group, ue, late, idea, grade, single):
                             ep2.idea_checkout(repo_owner)
                             click.secho('Project %s loaded into IntelliJ Idea' % repo_owner, fg='green')
 
+                    ep2.create_ex_test_symlink(repo_owner, team.name)
+
                     points = click.prompt("Points", type=click.IntRange(0, 4), show_choices=True)
                     remarks = click.prompt("Remarks", default='', show_default=True)
                     feedback = click.prompt("Feedback", default='', show_default=True)
@@ -525,6 +529,15 @@ def prompt_manual(prompt:str, suffix:str, type:click.Choice, show_default: bool,
     return Ok(prompt_result)
 
 
+class ExTestCheckoutInfo:
+    student_id: str
+    team: str
+
+    def __init__(self, student_id: str, team: str):
+        self.student_id = student_id
+        self.team = team
+
+
 @cli.command()
 @click.option("--group", required=True, prompt=True, help='name of the group')
 @click.option("--ue", required=True, prompt=True, help='number of the exercise, WITHOUT leading zero', type=click.INT)
@@ -552,6 +565,9 @@ def checkout(ctx, group, ue: int):
         click.secho("Could not find submissions file, please run 'submission' first or pull the tutor repo", fg='red')
         exit(1)
 
+    teams: {str} = set()
+    checkout_info: [ExTestCheckoutInfo] = []
+
     with open(csv_path, 'r') as csvfile:
         reader = csv.DictReader(csvfile, ex_test_csv_fieldnames(), KEY_INVALID, strict=True)
 
@@ -569,6 +585,8 @@ def checkout(ctx, group, ue: int):
                 click.secho('Malformed file: %s. Missing column(s)!' % csv_path, fg='red')
                 exit(1)
 
+            teams.add(row[KEY_EX_TEST_TEAM])
+
             if row[KEY_EX_TEST_ROLE] == 'o':
                 student_id = verify_and_normalize_student_id(row[KEY_STUDENT_ID])
 
@@ -579,16 +597,32 @@ def checkout(ctx, group, ue: int):
                 else:
                     student_id = student_id.ok()
 
-                click.echo('Checking out repository {} for team {}'.format(student_id, row[KEY_EX_TEST_TEAM]))
+                checkout_info += [ExTestCheckoutInfo(student_id, row[KEY_EX_TEST_TEAM])]
 
-                current_uebung = os.path.join(uebung_path, student_id)
-                if ep2.clone_or_update(current_uebung, ep2.ue_repo(student_id), TAG_NAME_EX_TEST_LATE % ue).is_err():
-                    clone_result = ep2.clone_or_update(current_uebung, ep2.ue_repo(student_id), TAG_NAME_EX_TEST % ue)
-                    if clone_result.is_err():
-                        click.secho("Could not checkout submission from {} for team {}, {}".format(student_id,
-                                                                                                   row[KEY_EX_TEST_TEAM],
-                                                                                                   clone_result.value),
-                                    fg='red')
+    teams_without_repo = list(filter(lambda team: all(chkout.team != team for chkout in checkout_info), teams))
+    if len(teams_without_repo) > 0:
+        click.secho('There are teams with no repository configured (role `o`):', fg='red', bold=True)
+        for team in teams_without_repo:
+            click.secho('    {}'.format(team), fg='red', bold='True')
+        exit(1)
+
+
+    with click.progressbar(checkout_info, label='Check out', item_show_func=lambda chkout: chkout.team if chkout is not None else '') as bar:
+        for chkout in bar:
+            student_id = chkout.student_id
+            team = chkout.team
+
+            current_uebung = os.path.join(uebung_path, student_id)
+            if ep2.clone_or_update(current_uebung, ep2.ue_repo(student_id), TAG_NAME_EX_TEST_LATE % ue).is_err():
+                clone_result = ep2.clone_or_update(current_uebung, ep2.ue_repo(student_id), TAG_NAME_EX_TEST % ue)
+                if clone_result.is_err():
+                    click.echo()
+                    click.secho("Could not checkout submission from {} for team {}, {}".format(student_id,
+                                                                                               row[KEY_EX_TEST_TEAM],
+                                                                                               clone_result.value),
+                                fg='red')
+
+            ep2.create_ex_test_symlink(student_id, team)
 
     click.echo('To grade the exercises run ' + click.style('%s grade' % EX_TEST_COMMAND_NAME, bold=True) + '.')
     click.echo('To load the projects into IntelliJ IDEA run ' + click.style('%s idea' % UTIL_COMMAND_NAME, bold=True) + '.')
@@ -614,6 +648,7 @@ def grade(ctx, team, points, ue, group, remarks, feedback):
     tmp_file = csv_file + ".tmp"  # tmp file for editing
 
     f_info.open_write(tmp_file, True)
+    matched = False
 
     with open(csv_file, 'r') as infile:
         with open(tmp_file, 'w') as outfile:
@@ -651,6 +686,7 @@ def grade(ctx, team, points, ue, group, remarks, feedback):
 
                 row[KEY_STUDENT_ID] = student_id
                 if row[KEY_EX_TEST_TEAM] == team:  # if team matches, update points
+                    matched = True
                     row[KEY_EX_TEST_POINTS] = points
                     if remarks is not None and remarks != '':
                         row[KEY_EX_TEST_REMARKS] = escape_csv_string(remarks)
@@ -658,6 +694,10 @@ def grade(ctx, team, points, ue, group, remarks, feedback):
                         row[KEY_EX_TEST_FEEDBACK] = escape_csv_string(feedback)
                 writer.writerow(row)  # write row
 
+    if not matched:
+        click.secho('Warning: Did not find team {} in submission list! Check the team name and the submissions list.'
+                    .format(team), fg='yellow', bold=True)
+
     f_info.open_write(csv_file, True)
     shutil.move(tmp_file, csv_file)  # replace old file with tmp file
     f_info.delete(tmp_file, True)
